#!/usr/bin/perl 

use diagnostics;
use warnings;
use strict;
use Getopt::Std;
use DBI;
use Config::IniFiles;
use XML::DOM;
use Net::IP qw(:PROC);
use POSIX qw(strftime);
my %opts;
#fonction de base format les mail est les sms
sub format_sms {
	my $txt = $_[0];
	my $dat =$_[1];
	#my $ip = new Net::IP ($dat->{device_device_ip_addr }) or die (Net::IP::Error());
	my $sip = ip_inttobin($dat->{device_ip_addr},$dat->{device_ip_version}); #on converti l'IP en binaire
	$sip = ip_bintoip($sip,$dat->{device_ip_version}); ##on l'a format en xxx.xxx.xxx.xxx
        $txt =~ s/\%node.group/$dat->{device_group_name}/gi;
	$txt =~ s/\%node.name/$dat->{device_name}/gi;
	$txt =~ s/\%node.ip/$sip/gi;
	$txt =~ s/\%node.description/$dat->{device_description}/gi;
	$txt =~ s/\%service.name/$dat->{service_name}/gi;
	$txt =~ s/\%service.parameter/$dat->{service_parameter}/gi;
	$txt =~ s/\%error.type/$dat->{error_type}/gi;
	$txt =~ s/\%error.name/$dat->{error_name}/gi;
	$txt =~ s/\%error.time/$dat->{happen_timestamp}/gi;
	$txt =~ s/\%error.message/$dat->{happen_message}/gi;
	$txt =~ s/\\n/\n/gi;
	if($dat->{error_type} eq 'critical')
	{
		$txt =~ s/\%color/#FF5D5D/gi;
	}
	elsif($dat->{error_type} eq 'warning')
	{
		$txt =~ s/\%color/#FFFD5E/gi;
	}
	elsif($dat->{error_type} eq 'unknwon')
	{
		$txt =~ s/\%color/#9F9E99/gi;
	}
	else
	{
		$txt =~ s/\%color/#FFFFFF/gi;
	}	
	return $txt;
#Noeud : %node.name : %node.ip \n Service: %service   ETAT: %error.type Heure: %error.time \n %error.message 
}
sub debug {
	if($opts{'v'})
	{
		print " $_[0]\n";
	}
}
my $url = "https://www.ovh.com/cgi-bin/sms/http2sms.cgi?account=sms-ed11075-1&login=esiacloud&password=TpOE5du9&from=ESIA&contentType=text/xml&";	
#my $url = "https://www.ovh.com/cgi-bin/sms/http2sms.cgi?account=sms-ed11075-1&login=esiacloud&password=TpOE5du9&from=0&contentType=text/xml&";	
my $config="/etc/esia/esia.conf";
#test des paramètres
if(!getopts( "e:t:c:h:m:v", \%opts ))
{
  print "Usage : send_sms.pl -e <id_error> -t <to num téléphone au format 0032xxxxx> -c <esia config file>\n";
  exit(1);
}
if($opts{'h'})
{
  print "Usage : send_sms.pl -e <id_error> -t <to num téléphone au format 0032xxxxx> -c <esia config file> [-v verbose mode] \n"; exit(1); 
}
if(!$opts{'e'})
{
   print "no error is defined\n"; exit(1); 
}
if($opts{'c'})
{
  $config=$opts{'c'};
}
if(!$opts{'t'})
{
  print "sms to not defined\n"; exit(1); 
}
my $to = $opts{'t'};
#chargement du fichier de configuration
debug("Chargement fichier de configuration");
my $cfg = new Config::IniFiles( -file => $config  )or die "esia config file not found (default: /etc/esia/esia.conf)";
#Connexion à la base de donnée
debug("Connection to database");
my $db = DBI -> connect($cfg->val( 'POSTGRESQL', 'PGSQL_connect' ),$cfg->val( 'POSTGRESQL', 'PGSQL_username' ),$cfg->val( 'POSTGRESQL', 'PGSQL_pwd' )) or die "Connection DB Failed";

#récupération des informations de configuration
debug("get configuration variable");
my $message;
my $hour_start;
my $hour_stop;
my $max_sms;
my $query = $db->prepare("SELECT config_key,config_value
                          FROM esia_configuration
                          WHERE config_key LIKE 'SMS_%';");	
$query->execute();
while (my @data = $query->fetchrow_array ) 
{
	if($data[0] eq 'SMS_FORMAT')
	{	
		$message = $data[1];
	}
	if($data[0] eq 'SMS_HOUR_START')
	{	
		$hour_start = $data[1];
	}
	if($data[0] eq 'SMS_HOUR_STOP')
	{	
		$hour_stop = $data[1];
	}
	if($data[0] eq 'SMS_MAX_PER_DAY')
	{	
		$max_sms = $data[1];
	}
}
#récupération des informations sur l'erreur
debug("get errors informations");
$query = $db->prepare("SELECT device_name,device_ip_addr,device_description,device_ip_version,device_description,error_name,error_type,to_char(happen_timestamp,'DD/MM/YYYY HH:MI:SS tz') AS happen_timestamp,happen_message,
				service_name,service_parameter,device_group_name 
                          FROM esia_error_happen
                          JOIN esia_device ON happen_device_id = device_id
                          JOIN esia_errors ON happen_error_id= error_id 
                          JOIN esia_services ON happen_service_id= service_id
                          WHERE happen_id=?");	
$query->execute($opts{'e'});
my $data = $query->fetchrow_hashref;
if($data)
{
	#formatage du message (voir fonction au début)
	$message = format_sms($message,$data);
}
#on finalise l'URL
$url = $url."to=$to&message=".$message;
$query->finish;

#récupération de l'heure pour la comparaison pour savoir si on envoit un SMS ou pas 
my $heure = strftime("%H", localtime);
if($heure >=  $hour_start && $heure <= $hour_stop)
{	
	debug("OK TEST HOUR $heure >  $hour_start && $heure < $hour_stop");
	# on vérifie qu'on a pas envoyé trop de SMS
	$query = $db->prepare("SELECT count(*) as nbr_sms FROM sms_send WHERE date(sms_date) = CURRENT_DATE;");	
	$query->execute();
	my @data = $query->fetchrow_array;
	$query->finish;
	debug("SMS number $data[0] < $max_sms (max per day)");
	if($data[0] < $max_sms)
	{
		debug("Send SMS");
		#on envoit le SMS
		my $parser = new XML::DOM::Parser();
		my $doc = $parser->parsefile ($url);
		my @content = $doc->getElementsByTagName("status");
		my $state = $content[0]->getFirstChild()->toString();	
		if($state eq '100')
		{
			debug("Send SMS success");
			#OK le sms a bien été envoyé. On insère les informations en DB
			$query = $db->prepare("INSERT INTO sms_send(sms_id, sms_to, sms_date, sms_message) VALUES (DEFAULT, ?, now(), ?);");
			my @num = split(',', $to);
			foreach my $numeros (@num)
			{
				$query->execute($numeros,$message);
			}	
			$query->finish;
			debug("insert SMS to database success");
		}
		else
		{
			debug("Send SMS failed");
			@content = $doc->getElementsByTagName("message");
			debug($content[0]->getFirstChild()->toString());
			
		}
	}
	else
	{
		debug("To much send SMS this day");
	}
}
else
{
	debug("TO LATE test $heure >  $hour_start && $heure < $hour_stop");
}
#si on veut une copie en mail
if($opts{'m'})
{
	debug("Send mail");
	my $ret = system('/usr/local/esia/plugins/alerting/send_mail_html.pl -e '.$opts{'e'}.' -t '.$opts{'m'}); 
}
$db->disconnect; 
debug("finish");
exit(0);
